# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._jobs_operations import JobsOperations
from ._providers_operations import ProvidersOperations
from ._storage_operations import StorageOperations
from ._quotas_operations import QuotasOperations

__all__ = [
    'JobsOperations',
    'ProvidersOperations',
    'StorageOperations',
    'QuotasOperations',
]
